@@ -19,11 +19,19 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
+import sys
 import threading
-import collections.abc
 import functools
+from enum import Enum
+
 import six
 
+if sys.version_info > (3, 3):
+    from collections.abc import Callable as abc_Callable, Mapping as abc_Mapping
+else:
+    from collections import Callable as abc_Callable, Mapping as abc_Mapping
+
+
 import hamcrest
 from hamcrest.core.base_matcher import BaseMatcher
 
@@ -40,16 +48,21 @@ class WrongApiUsage(Exception):
     pass
 
 
-class Constant(str):
+class Constant(str, Enum):
+    ANY_ARG = 'ANY_ARG'
+    UNSPECIFIED = 'UNSPECIFIED'
+
+    def __str__(self):
+        return self.value
+
     def __repr__(self):
         return str(self)
 
-    def __eq__(self, other):
-        return self is other
+    def is_in(self, container):
+        return any(item is self for item in container)
 
-
-ANY_ARG = Constant('ANY_ARG')
-UNSPECIFIED = Constant('UNSPECIFIED')
+ANY_ARG = Constant.ANY_ARG
+UNSPECIFIED = Constant.UNSPECIFIED
 
 
 def add_indent(text, indent=0):
@@ -88,6 +101,10 @@ class Observable(object):
         for ob in self.observers:
             ob(*args, **kargs)
 
+    def _apply_deactivation(self, double):
+        if double._deactivate:
+            double._setting_up = self.double._deactivate = False
+
 
 class Method(Observable):
     def __init__(self, double, name):
@@ -106,9 +123,7 @@ class Method(Observable):
             self._event.set()
             self.notify(*args, **kargs)
 
-        if self.double._deactivate:
-            self.double._setting_up = self.double._deactivate = False
-
+        self._apply_deactivation(self.double)
         return retval
 
     def _create_invocation(self, args, kargs):
@@ -184,11 +199,11 @@ class Invocation(object):
         return Invocation(double, name, InvocationContext(*args, **kargs))
 
     def delegates(self, delegate):
-        if isinstance(delegate, collections.abc.Callable):
+        if isinstance(delegate, abc_Callable):
             self.__delegate = delegate
             return
 
-        if isinstance(delegate, collections.abc.Mapping):
+        if isinstance(delegate, abc_Mapping):
             self.__delegate = delegate.get
             return
 
@@ -267,7 +282,7 @@ class InvocationContext(object):
             raise ValueError
 
         try:
-#            if args.index(ANY_ARG) != len(args) - 1:
+            # if args.index(ANY_ARG) != len(args) - 1:
             if find_ANY_ARG(args) != len(args) - 1:
                 raise WrongApiUsage(ANY_ARG_MUST_BE_LAST + ANY_ARG_DOC)
 
@@ -276,7 +291,7 @@ class InvocationContext(object):
         except ValueError:
             pass
 
-        if ANY_ARG in kargs.values():
+        if ANY_ARG.is_in(kargs.values()):
             raise WrongApiUsage(ANY_ARG_CAN_BE_KARG + ANY_ARG_DOC)
 
     def apply_on(self, method):
@@ -343,7 +358,7 @@ class InvocationContext(object):
         return retval
 
     def matches(self, other):
-        if ANY_ARG in self.args:
+        if ANY_ARG.is_in(self.args):
             matcher, actual = self, other
         else:
             matcher, actual = other, self
@@ -380,7 +395,7 @@ class InvocationContext(object):
         return retval
 
     def __lt__(self, other):
-        if ANY_ARG in other.args or self.args < other.args:
+        if ANY_ARG.is_in(other.args) or self.args < other.args:
             return True
 
         return sorted(self.kargs.items()) < sorted(other.kargs.items())
@@ -473,7 +488,9 @@ class Property(property, Observable):
         if not self.double._setting_up:
             self.notify()
 
-        return self.manage(PropertyGet(self.double, self.key))
+        property_get = self.manage(PropertyGet(self.double, self.key))
+        self._apply_deactivation(self.double)
+        return property_get
 
     def set_value(self, obj, value):
         prop = self.double._proxy.get_class_attr(self.key)
@@ -487,6 +504,8 @@ class Property(property, Observable):
         else:
             self.notify(value)
 
+        self._apply_deactivation(self.double)
+
 
 class AttributeFactory(object):
     """Create double methods, properties or attributes from collaborator"""
@@ -499,6 +518,7 @@ class AttributeFactory(object):
         # -- python3 --
         method             = Method,
         function           = Method,
+        _tuplegetter       = Property,
         )
 
     @classmethod
